# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from .._models import BaseModel

__all__ = ["RecommendationGenerateResponse"]


class RecommendationGenerateResponse(BaseModel):
    snapshot_id: str



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Required, TypedDict

from .._types import FileTypes

__all__ = ["DatasetCreateFromJSONLParams"]


class DatasetCreateFromJSONLParams(TypedDict, total=False):
    file: Required[FileTypes]
    """Required JSONL upload.

    Each line should be a JSON object containing a "messages" array
    (system/user/assistant) used to seed the dataset.
    """

    name: Required[str]
    """Human-readable name shown in the dashboard once the dataset is created."""

    project_id: Required[str]
    """Project ID that will own the dataset. Must match a project you created."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from .._models import BaseModel

__all__ = ["FinetuningCreateResponse"]


class FinetuningCreateResponse(BaseModel):
    job_id: str



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional
from typing_extensions import Literal

from pydantic import Field as FieldInfo

from .._models import BaseModel

__all__ = [
    "ProjectGetTreeResponse",
    "Project",
    "ProjectChild",
    "ProjectChildMetadata",
    "ProjectChildChild",
    "ProjectChildChildMetadata",
    "ProjectChildChildChild",
    "ProjectChildChildChildMetadata",
    "ProjectChildChildChildChild",
    "ProjectChildChildChildChildMetadata",
]


class ProjectChildMetadata(BaseModel):
    id: str

    datapoints_count: float


class ProjectChildChildMetadata(BaseModel):
    id: str

    created_at: str

    training_count: float

    validation_count: float


class ProjectChildChildChildMetadata(BaseModel):
    id: str

    reasoning: bool


class ProjectChildChildChildChildMetadata(BaseModel):
    id: str

    base_model_id: str

    experiment_number: float

    api_model_id: Optional[str] = FieldInfo(alias="model_id", default=None)


class ProjectChildChildChildChild(BaseModel):
    id: str

    label: str

    metadata: ProjectChildChildChildChildMetadata

    status: Literal["pending", "queued", "running", "deploying", "succeeded", "failed", "deleted"]

    type: Literal["experiment"]


class ProjectChildChildChild(BaseModel):
    id: str

    label: str

    metadata: ProjectChildChildChildMetadata

    status: Literal["processing", "completed", "failed"]

    type: Literal["finetuning-job"]

    children: Optional[List[ProjectChildChildChildChild]] = None


class ProjectChildChild(BaseModel):
    id: str

    label: str

    metadata: ProjectChildChildMetadata

    type: Literal["snapshot"]

    children: Optional[List[ProjectChildChildChild]] = None


class ProjectChild(BaseModel):
    id: str

    label: str

    metadata: ProjectChildMetadata

    status: Literal["processing", "completed", "failed"]

    type: Literal["dataset"]

    children: Optional[List[ProjectChildChild]] = None


class Project(BaseModel):
    id: str

    name: str

    type: Literal["project"]

    children: Optional[List[ProjectChild]] = None


class ProjectGetTreeResponse(BaseModel):
    project: Project



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from .chat import (
    ChatResource,
    AsyncChatResource,
    ChatResourceWithRawResponse,
    AsyncChatResourceWithRawResponse,
    ChatResourceWithStreamingResponse,
    AsyncChatResourceWithStreamingResponse,
)
from .models import (
    ModelsResource,
    AsyncModelsResource,
    ModelsResourceWithRawResponse,
    AsyncModelsResourceWithRawResponse,
    ModelsResourceWithStreamingResponse,
    AsyncModelsResourceWithStreamingResponse,
)
from .datasets import (
    DatasetsResource,
    AsyncDatasetsResource,
    DatasetsResourceWithRawResponse,
    AsyncDatasetsResourceWithRawResponse,
    DatasetsResourceWithStreamingResponse,
    AsyncDatasetsResourceWithStreamingResponse,
)
from .projects import (
    ProjectsResource,
    AsyncProjectsResource,
    ProjectsResourceWithRawResponse,
    AsyncProjectsResourceWithRawResponse,
    ProjectsResourceWithStreamingResponse,
    AsyncProjectsResourceWithStreamingResponse,
)
from .snapshots import (
    SnapshotsResource,
    AsyncSnapshotsResource,
    SnapshotsResourceWithRawResponse,
    AsyncSnapshotsResourceWithRawResponse,
    SnapshotsResourceWithStreamingResponse,
    AsyncSnapshotsResourceWithStreamingResponse,
)
from .finetuning import (
    FinetuningResource,
    AsyncFinetuningResource,
    FinetuningResourceWithRawResponse,
    AsyncFinetuningResourceWithRawResponse,
    FinetuningResourceWithStreamingResponse,
    AsyncFinetuningResourceWithStreamingResponse,
)
from .recommendations import (
    RecommendationsResource,
    AsyncRecommendationsResource,
    RecommendationsResourceWithRawResponse,
    AsyncRecommendationsResourceWithRawResponse,
    RecommendationsResourceWithStreamingResponse,
    AsyncRecommendationsResourceWithStreamingResponse,
)

__all__ = [
    "ChatResource",
    "AsyncChatResource",
    "ChatResourceWithRawResponse",
    "AsyncChatResourceWithRawResponse",
    "ChatResourceWithStreamingResponse",
    "AsyncChatResourceWithStreamingResponse",
    "ModelsResource",
    "AsyncModelsResource",
    "ModelsResourceWithRawResponse",
    "AsyncModelsResourceWithRawResponse",
    "ModelsResourceWithStreamingResponse",
    "AsyncModelsResourceWithStreamingResponse",
    "ProjectsResource",
    "AsyncProjectsResource",
    "ProjectsResourceWithRawResponse",
    "AsyncProjectsResourceWithRawResponse",
    "ProjectsResourceWithStreamingResponse",
    "AsyncProjectsResourceWithStreamingResponse",
    "DatasetsResource",
    "AsyncDatasetsResource",
    "DatasetsResourceWithRawResponse",
    "AsyncDatasetsResourceWithRawResponse",
    "DatasetsResourceWithStreamingResponse",
    "AsyncDatasetsResourceWithStreamingResponse",
    "SnapshotsResource",
    "AsyncSnapshotsResource",
    "SnapshotsResourceWithRawResponse",
    "AsyncSnapshotsResourceWithRawResponse",
    "SnapshotsResourceWithStreamingResponse",
    "AsyncSnapshotsResourceWithStreamingResponse",
    "RecommendationsResource",
    "AsyncRecommendationsResource",
    "RecommendationsResourceWithRawResponse",
    "AsyncRecommendationsResourceWithRawResponse",
    "RecommendationsResourceWithStreamingResponse",
    "AsyncRecommendationsResourceWithStreamingResponse",
    "FinetuningResource",
    "AsyncFinetuningResource",
    "FinetuningResourceWithRawResponse",
    "AsyncFinetuningResourceWithRawResponse",
    "FinetuningResourceWithStreamingResponse",
    "AsyncFinetuningResourceWithStreamingResponse",
]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional
from typing_extensions import Literal

from .._models import BaseModel

__all__ = [
    "RecommendationGetResponse",
    "RecommendedExperiment",
    "RecommendedModel",
    "RecommendedModelFullHyperparameters",
    "RecommendedModelLoraHyperparameters",
]


class RecommendedExperiment(BaseModel):
    base_model_id: str

    batch_size: int

    learning_rate_multiplier: float

    lora: bool

    n_epochs: int

    reason_for_recommendation: Optional[str] = None

    recommended: bool


class RecommendedModelFullHyperparameters(BaseModel):
    batch_size: int

    learning_rate_multiplier: float

    n_epochs: int


class RecommendedModelLoraHyperparameters(BaseModel):
    batch_size: int

    learning_rate_multiplier: float

    n_epochs: int


class RecommendedModel(BaseModel):
    base_model_id: str

    full_hyperparameters: Optional[RecommendedModelFullHyperparameters] = None

    lora_hyperparameters: Optional[RecommendedModelLoraHyperparameters] = None

    reason_for_recommendation: Optional[str] = None

    recommended: bool


class RecommendationGetResponse(BaseModel):
    recommended_experiments: Optional[List[RecommendedExperiment]] = None

    recommended_models: Optional[List[RecommendedModel]] = None

    snapshot_id: str

    status: Literal["processing", "completed", "failed"]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import httpx

from ..types import project_create_params
from .._types import Body, Query, Headers, NotGiven, not_given
from .._utils import maybe_transform, async_maybe_transform
from .._compat import cached_property
from .._resource import SyncAPIResource, AsyncAPIResource
from .._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from .._base_client import make_request_options
from ..types.project_list_response import ProjectListResponse
from ..types.project_create_response import ProjectCreateResponse
from ..types.project_get_tree_response import ProjectGetTreeResponse

__all__ = ["ProjectsResource", "AsyncProjectsResource"]


class ProjectsResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> ProjectsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/premAI-io/prem-py-sdk#accessing-raw-response-data-eg-headers
        """
        return ProjectsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> ProjectsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/premAI-io/prem-py-sdk#with_streaming_response
        """
        return ProjectsResourceWithStreamingResponse(self)

    def create(
        self,
        *,
        goal: str,
        name: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> ProjectCreateResponse:
        """
        Create a new project

        Args:
          goal: The goal or purpose of this project

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._post(
            "/api/v1/public/projects/create",
            body=maybe_transform(
                {
                    "goal": goal,
                    "name": name,
                },
                project_create_params.ProjectCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=ProjectCreateResponse,
        )

    def list(
        self,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> ProjectListResponse:
        """List all projects for the authenticated user"""
        return self._get(
            "/api/v1/public/projects",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=ProjectListResponse,
        )

    def get_tree(
        self,
        project_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> ProjectGetTreeResponse:
        """
        Get complete project tree with datasets, snapshots, and finetuning jobs

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not project_id:
            raise ValueError(f"Expected a non-empty value for `project_id` but received {project_id!r}")
        return self._get(
            f"/api/v1/public/projects/{project_id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=ProjectGetTreeResponse,
        )


class AsyncProjectsResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncProjectsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/premAI-io/prem-py-sdk#accessing-raw-response-data-eg-headers
        """
        return AsyncProjectsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncProjectsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/premAI-io/prem-py-sdk#with_streaming_response
        """
        return AsyncProjectsResourceWithStreamingResponse(self)

    async def create(
        self,
        *,
        goal: str,
        name: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> ProjectCreateResponse:
        """
        Create a new project

        Args:
          goal: The goal or purpose of this project

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._post(
            "/api/v1/public/projects/create",
            body=await async_maybe_transform(
                {
                    "goal": goal,
                    "name": name,
                },
                project_create_params.ProjectCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=ProjectCreateResponse,
        )

    async def list(
        self,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> ProjectListResponse:
        """List all projects for the authenticated user"""
        return await self._get(
            "/api/v1/public/projects",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=ProjectListResponse,
        )

    async def get_tree(
        self,
        project_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> ProjectGetTreeResponse:
        """
        Get complete project tree with datasets, snapshots, and finetuning jobs

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not project_id:
            raise ValueError(f"Expected a non-empty value for `project_id` but received {project_id!r}")
        return await self._get(
            f"/api/v1/public/projects/{project_id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=ProjectGetTreeResponse,
        )


class ProjectsResourceWithRawResponse:
    def __init__(self, projects: ProjectsResource) -> None:
        self._projects = projects

        self.create = to_raw_response_wrapper(
            projects.create,
        )
        self.list = to_raw_response_wrapper(
            projects.list,
        )
        self.get_tree = to_raw_response_wrapper(
            projects.get_tree,
        )


class AsyncProjectsResourceWithRawResponse:
    def __init__(self, projects: AsyncProjectsResource) -> None:
        self._projects = projects

        self.create = async_to_raw_response_wrapper(
            projects.create,
        )
        self.list = async_to_raw_response_wrapper(
            projects.list,
        )
        self.get_tree = async_to_raw_response_wrapper(
            projects.get_tree,
        )


class ProjectsResourceWithStreamingResponse:
    def __init__(self, projects: ProjectsResource) -> None:
        self._projects = projects

        self.create = to_streamed_response_wrapper(
            projects.create,
        )
        self.list = to_streamed_response_wrapper(
            projects.list,
        )
        self.get_tree = to_streamed_response_wrapper(
            projects.get_tree,
        )


class AsyncProjectsResourceWithStreamingResponse:
    def __init__(self, projects: AsyncProjectsResource) -> None:
        self._projects = projects

        self.create = async_to_streamed_response_wrapper(
            projects.create,
        )
        self.list = async_to_streamed_response_wrapper(
            projects.list,
        )
        self.get_tree = async_to_streamed_response_wrapper(
            projects.get_tree,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from .._models import BaseModel

__all__ = ["ProjectCreateResponse"]


class ProjectCreateResponse(BaseModel):
    project_id: str



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Mapping, Iterable, Optional, cast
from typing_extensions import Literal

import httpx

from ..types import dataset_add_datapoint_params, dataset_create_synthetic_params, dataset_create_from_jsonl_params
from .._types import Body, Omit, Query, Headers, NotGiven, FileTypes, SequenceNotStr, omit, not_given
from .._utils import extract_files, maybe_transform, deepcopy_minimal, async_maybe_transform
from .._compat import cached_property
from .._resource import SyncAPIResource, AsyncAPIResource
from .._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from .._base_client import make_request_options
from ..types.dataset_get_response import DatasetGetResponse
from ..types.dataset_add_datapoint_response import DatasetAddDatapointResponse
from ..types.dataset_create_synthetic_response import DatasetCreateSyntheticResponse
from ..types.dataset_create_from_jsonl_response import DatasetCreateFromJSONLResponse

__all__ = ["DatasetsResource", "AsyncDatasetsResource"]


class DatasetsResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> DatasetsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/premAI-io/prem-py-sdk#accessing-raw-response-data-eg-headers
        """
        return DatasetsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> DatasetsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/premAI-io/prem-py-sdk#with_streaming_response
        """
        return DatasetsResourceWithStreamingResponse(self)

    def add_datapoint(
        self,
        dataset_id: str,
        *,
        messages: Iterable[dataset_add_datapoint_params.Message],
        bucket: Literal["uncategorized", "training", "validation"] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> DatasetAddDatapointResponse:
        """
        Add a single datapoint to a dataset.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not dataset_id:
            raise ValueError(f"Expected a non-empty value for `dataset_id` but received {dataset_id!r}")
        return self._post(
            f"/api/v1/datasets/{dataset_id}/addDatapoint",
            body=maybe_transform(
                {
                    "messages": messages,
                    "bucket": bucket,
                },
                dataset_add_datapoint_params.DatasetAddDatapointParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=DatasetAddDatapointResponse,
        )

    def create_from_jsonl(
        self,
        *,
        file: FileTypes,
        name: str,
        project_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> DatasetCreateFromJSONLResponse:
        """Create dataset from JSONL file

        Args:
          file: Required JSONL upload.

        Each line should be a JSON object containing a "messages"
              array (system/user/assistant) used to seed the dataset.

          name: Human-readable name shown in the dashboard once the dataset is created.

          project_id: Project ID that will own the dataset. Must match a project you created.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        body = deepcopy_minimal(
            {
                "file": file,
                "name": name,
                "project_id": project_id,
            }
        )
        files = extract_files(cast(Mapping[str, object], body), paths=[["file"]])
        # It should be noted that the actual Content-Type header that will be
        # sent to the server will contain a `boundary` parameter, e.g.
        # multipart/form-data; boundary=---abc--
        extra_headers = {"Content-Type": "multipart/form-data", **(extra_headers or {})}
        return self._post(
            "/api/v1/public/datasets/create-from-jsonl",
            body=maybe_transform(body, dataset_create_from_jsonl_params.DatasetCreateFromJSONLParams),
            files=files,
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=DatasetCreateFromJSONLResponse,
        )

    def create_synthetic(
        self,
        *,
        name: str,
        pairs_to_generate: int,
        project_id: str,
        answer_format: str | Omit = omit,
        example_answers: SequenceNotStr[str] | Omit = omit,
        example_questions: SequenceNotStr[str] | Omit = omit,
        files: SequenceNotStr[FileTypes] | Omit = omit,
        question_format: str | Omit = omit,
        rules: SequenceNotStr[str] | Omit = omit,
        temperature: Optional[float] | Omit = omit,
        website_urls: SequenceNotStr[str] | Omit = omit,
        youtube_urls: SequenceNotStr[str] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> DatasetCreateSyntheticResponse:
        """
        Create synthetic dataset

        Args:
          answer_format: Answer format template

          example_answers: Example answers

          example_questions: Example questions

          files: Optional: PDF, DOCX, etc.

          question_format: Question format template

          rules: Array of rules and constraints

          temperature: 0.0-1.0, controls randomness

          website_urls: Array of website URLs

          youtube_urls: Array of YouTube URLs

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        body = deepcopy_minimal(
            {
                "name": name,
                "pairs_to_generate": pairs_to_generate,
                "project_id": project_id,
                "answer_format": answer_format,
                "example_answers": example_answers,
                "example_questions": example_questions,
                "files": files,
                "question_format": question_format,
                "rules": rules,
                "temperature": temperature,
                "website_urls": website_urls,
                "youtube_urls": youtube_urls,
            }
        )
        extracted_files = extract_files(cast(Mapping[str, object], body), paths=[["files", "<array>"]])
        # It should be noted that the actual Content-Type header that will be
        # sent to the server will contain a `boundary` parameter, e.g.
        # multipart/form-data; boundary=---abc--
        extra_headers = {"Content-Type": "multipart/form-data", **(extra_headers or {})}
        return self._post(
            "/api/v1/public/datasets/create-synthetic",
            body=maybe_transform(body, dataset_create_synthetic_params.DatasetCreateSyntheticParams),
            files=extracted_files,
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=DatasetCreateSyntheticResponse,
        )

    def get(
        self,
        dataset_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> DatasetGetResponse:
        """
        Get dataset status

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not dataset_id:
            raise ValueError(f"Expected a non-empty value for `dataset_id` but received {dataset_id!r}")
        return self._get(
            f"/api/v1/public/datasets/{dataset_id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=DatasetGetResponse,
        )


class AsyncDatasetsResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncDatasetsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/premAI-io/prem-py-sdk#accessing-raw-response-data-eg-headers
        """
        return AsyncDatasetsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncDatasetsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/premAI-io/prem-py-sdk#with_streaming_response
        """
        return AsyncDatasetsResourceWithStreamingResponse(self)

    async def add_datapoint(
        self,
        dataset_id: str,
        *,
        messages: Iterable[dataset_add_datapoint_params.Message],
        bucket: Literal["uncategorized", "training", "validation"] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> DatasetAddDatapointResponse:
        """
        Add a single datapoint to a dataset.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not dataset_id:
            raise ValueError(f"Expected a non-empty value for `dataset_id` but received {dataset_id!r}")
        return await self._post(
            f"/api/v1/datasets/{dataset_id}/addDatapoint",
            body=await async_maybe_transform(
                {
                    "messages": messages,
                    "bucket": bucket,
                },
                dataset_add_datapoint_params.DatasetAddDatapointParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=DatasetAddDatapointResponse,
        )

    async def create_from_jsonl(
        self,
        *,
        file: FileTypes,
        name: str,
        project_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> DatasetCreateFromJSONLResponse:
        """Create dataset from JSONL file

        Args:
          file: Required JSONL upload.

        Each line should be a JSON object containing a "messages"
              array (system/user/assistant) used to seed the dataset.

          name: Human-readable name shown in the dashboard once the dataset is created.

          project_id: Project ID that will own the dataset. Must match a project you created.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        body = deepcopy_minimal(
            {
                "file": file,
                "name": name,
                "project_id": project_id,
            }
        )
        files = extract_files(cast(Mapping[str, object], body), paths=[["file"]])
        # It should be noted that the actual Content-Type header that will be
        # sent to the server will contain a `boundary` parameter, e.g.
        # multipart/form-data; boundary=---abc--
        extra_headers = {"Content-Type": "multipart/form-data", **(extra_headers or {})}
        return await self._post(
            "/api/v1/public/datasets/create-from-jsonl",
            body=await async_maybe_transform(body, dataset_create_from_jsonl_params.DatasetCreateFromJSONLParams),
            files=files,
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=DatasetCreateFromJSONLResponse,
        )

    async def create_synthetic(
        self,
        *,
        name: str,
        pairs_to_generate: int,
        project_id: str,
        answer_format: str | Omit = omit,
        example_answers: SequenceNotStr[str] | Omit = omit,
        example_questions: SequenceNotStr[str] | Omit = omit,
        files: SequenceNotStr[FileTypes] | Omit = omit,
        question_format: str | Omit = omit,
        rules: SequenceNotStr[str] | Omit = omit,
        temperature: Optional[float] | Omit = omit,
        website_urls: SequenceNotStr[str] | Omit = omit,
        youtube_urls: SequenceNotStr[str] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> DatasetCreateSyntheticResponse:
        """
        Create synthetic dataset

        Args:
          answer_format: Answer format template

          example_answers: Example answers

          example_questions: Example questions

          files: Optional: PDF, DOCX, etc.

          question_format: Question format template

          rules: Array of rules and constraints

          temperature: 0.0-1.0, controls randomness

          website_urls: Array of website URLs

          youtube_urls: Array of YouTube URLs

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        body = deepcopy_minimal(
            {
                "name": name,
                "pairs_to_generate": pairs_to_generate,
                "project_id": project_id,
                "answer_format": answer_format,
                "example_answers": example_answers,
                "example_questions": example_questions,
                "files": files,
                "question_format": question_format,
                "rules": rules,
                "temperature": temperature,
                "website_urls": website_urls,
                "youtube_urls": youtube_urls,
            }
        )
        extracted_files = extract_files(cast(Mapping[str, object], body), paths=[["files", "<array>"]])
        # It should be noted that the actual Content-Type header that will be
        # sent to the server will contain a `boundary` parameter, e.g.
        # multipart/form-data; boundary=---abc--
        extra_headers = {"Content-Type": "multipart/form-data", **(extra_headers or {})}
        return await self._post(
            "/api/v1/public/datasets/create-synthetic",
            body=await async_maybe_transform(body, dataset_create_synthetic_params.DatasetCreateSyntheticParams),
            files=extracted_files,
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=DatasetCreateSyntheticResponse,
        )

    async def get(
        self,
        dataset_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> DatasetGetResponse:
        """
        Get dataset status

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not dataset_id:
            raise ValueError(f"Expected a non-empty value for `dataset_id` but received {dataset_id!r}")
        return await self._get(
            f"/api/v1/public/datasets/{dataset_id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=DatasetGetResponse,
        )


class DatasetsResourceWithRawResponse:
    def __init__(self, datasets: DatasetsResource) -> None:
        self._datasets = datasets

        self.add_datapoint = to_raw_response_wrapper(
            datasets.add_datapoint,
        )
        self.create_from_jsonl = to_raw_response_wrapper(
            datasets.create_from_jsonl,
        )
        self.create_synthetic = to_raw_response_wrapper(
            datasets.create_synthetic,
        )
        self.get = to_raw_response_wrapper(
            datasets.get,
        )


class AsyncDatasetsResourceWithRawResponse:
    def __init__(self, datasets: AsyncDatasetsResource) -> None:
        self._datasets = datasets

        self.add_datapoint = async_to_raw_response_wrapper(
            datasets.add_datapoint,
        )
        self.create_from_jsonl = async_to_raw_response_wrapper(
            datasets.create_from_jsonl,
        )
        self.create_synthetic = async_to_raw_response_wrapper(
            datasets.create_synthetic,
        )
        self.get = async_to_raw_response_wrapper(
            datasets.get,
        )


class DatasetsResourceWithStreamingResponse:
    def __init__(self, datasets: DatasetsResource) -> None:
        self._datasets = datasets

        self.add_datapoint = to_streamed_response_wrapper(
            datasets.add_datapoint,
        )
        self.create_from_jsonl = to_streamed_response_wrapper(
            datasets.create_from_jsonl,
        )
        self.create_synthetic = to_streamed_response_wrapper(
            datasets.create_synthetic,
        )
        self.get = to_streamed_response_wrapper(
            datasets.get,
        )


class AsyncDatasetsResourceWithStreamingResponse:
    def __init__(self, datasets: AsyncDatasetsResource) -> None:
        self._datasets = datasets

        self.add_datapoint = async_to_streamed_response_wrapper(
            datasets.add_datapoint,
        )
        self.create_from_jsonl = async_to_streamed_response_wrapper(
            datasets.create_from_jsonl,
        )
        self.create_synthetic = async_to_streamed_response_wrapper(
            datasets.create_synthetic,
        )
        self.get = async_to_streamed_response_wrapper(
            datasets.get,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional
from typing_extensions import Literal

from pydantic import Field as FieldInfo

from .._models import BaseModel

__all__ = ["FinetuningGetResponse", "Experiment"]


class Experiment(BaseModel):
    id: str

    base_model_id: str

    batch_size: int

    experiment_number: int

    learning_rate_multiplier: float

    lora: bool

    n_epochs: int

    status: Literal["pending", "queued", "running", "deploying", "succeeded", "failed", "deleted"]

    api_model_id: Optional[str] = FieldInfo(alias="model_id", default=None)


class FinetuningGetResponse(BaseModel):
    id: str

    experiments: List[Experiment]

    name: str

    snapshot_id: str

    status: Literal["processing", "completed", "failed"]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from .._models import BaseModel

__all__ = ["ModelLoadResponse"]


class ModelLoadResponse(BaseModel):
    message: str



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Required, TypedDict

__all__ = ["ProjectCreateParams"]


class ProjectCreateParams(TypedDict, total=False):
    goal: Required[str]
    """The goal or purpose of this project"""

    name: Required[str]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Iterable
from typing_extensions import Literal, Required, TypedDict

__all__ = ["DatasetAddDatapointParams", "Message"]


class DatasetAddDatapointParams(TypedDict, total=False):
    messages: Required[Iterable[Message]]

    bucket: Literal["uncategorized", "training", "validation"]


class Message(TypedDict, total=False):
    content: Required[str]

    role: Required[Literal["user", "assistant", "system"]]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import os
from typing import Any, cast

import pytest

from premai import PremAI, AsyncPremAI
from tests.utils import assert_matches_type
from premai.types import (
    DatasetGetResponse,
    DatasetAddDatapointResponse,
    DatasetCreateFromJSONLResponse,
    DatasetCreateSyntheticResponse,
)

base_url = os.environ.get("TEST_API_BASE_URL", "http://127.0.0.1:4010")


class TestDatasets:
    parametrize = pytest.mark.parametrize("client", [False, True], indirect=True, ids=["loose", "strict"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_add_datapoint(self, client: PremAI) -> None:
        dataset = client.datasets.add_datapoint(
            dataset_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            messages=[
                {
                    "content": "content",
                    "role": "user",
                }
            ],
        )
        assert_matches_type(DatasetAddDatapointResponse, dataset, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_add_datapoint_with_all_params(self, client: PremAI) -> None:
        dataset = client.datasets.add_datapoint(
            dataset_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            messages=[
                {
                    "content": "content",
                    "role": "user",
                }
            ],
            bucket="uncategorized",
        )
        assert_matches_type(DatasetAddDatapointResponse, dataset, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_raw_response_add_datapoint(self, client: PremAI) -> None:
        response = client.datasets.with_raw_response.add_datapoint(
            dataset_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            messages=[
                {
                    "content": "content",
                    "role": "user",
                }
            ],
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        dataset = response.parse()
        assert_matches_type(DatasetAddDatapointResponse, dataset, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_streaming_response_add_datapoint(self, client: PremAI) -> None:
        with client.datasets.with_streaming_response.add_datapoint(
            dataset_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            messages=[
                {
                    "content": "content",
                    "role": "user",
                }
            ],
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            dataset = response.parse()
            assert_matches_type(DatasetAddDatapointResponse, dataset, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_path_params_add_datapoint(self, client: PremAI) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `dataset_id` but received ''"):
            client.datasets.with_raw_response.add_datapoint(
                dataset_id="",
                messages=[
                    {
                        "content": "content",
                        "role": "user",
                    }
                ],
            )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_create_from_jsonl(self, client: PremAI) -> None:
        dataset = client.datasets.create_from_jsonl(
            file=b"raw file contents",
            name="x",
            project_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        )
        assert_matches_type(DatasetCreateFromJSONLResponse, dataset, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_raw_response_create_from_jsonl(self, client: PremAI) -> None:
        response = client.datasets.with_raw_response.create_from_jsonl(
            file=b"raw file contents",
            name="x",
            project_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        dataset = response.parse()
        assert_matches_type(DatasetCreateFromJSONLResponse, dataset, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_streaming_response_create_from_jsonl(self, client: PremAI) -> None:
        with client.datasets.with_streaming_response.create_from_jsonl(
            file=b"raw file contents",
            name="x",
            project_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            dataset = response.parse()
            assert_matches_type(DatasetCreateFromJSONLResponse, dataset, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_create_synthetic(self, client: PremAI) -> None:
        dataset = client.datasets.create_synthetic(
            name="x",
            pairs_to_generate=1,
            project_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        )
        assert_matches_type(DatasetCreateSyntheticResponse, dataset, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_create_synthetic_with_all_params(self, client: PremAI) -> None:
        dataset = client.datasets.create_synthetic(
            name="x",
            pairs_to_generate=1,
            project_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            answer_format="answer_format",
            example_answers=["string"],
            example_questions=["string"],
            files=[b"raw file contents"],
            question_format="question_format",
            rules=["string"],
            temperature=0,
            website_urls=["https://example.com"],
            youtube_urls=["https://example.com"],
        )
        assert_matches_type(DatasetCreateSyntheticResponse, dataset, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_raw_response_create_synthetic(self, client: PremAI) -> None:
        response = client.datasets.with_raw_response.create_synthetic(
            name="x",
            pairs_to_generate=1,
            project_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        dataset = response.parse()
        assert_matches_type(DatasetCreateSyntheticResponse, dataset, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_streaming_response_create_synthetic(self, client: PremAI) -> None:
        with client.datasets.with_streaming_response.create_synthetic(
            name="x",
            pairs_to_generate=1,
            project_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            dataset = response.parse()
            assert_matches_type(DatasetCreateSyntheticResponse, dataset, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_get(self, client: PremAI) -> None:
        dataset = client.datasets.get(
            "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        )
        assert_matches_type(DatasetGetResponse, dataset, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_raw_response_get(self, client: PremAI) -> None:
        response = client.datasets.with_raw_response.get(
            "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        dataset = response.parse()
        assert_matches_type(DatasetGetResponse, dataset, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_streaming_response_get(self, client: PremAI) -> None:
        with client.datasets.with_streaming_response.get(
            "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            dataset = response.parse()
            assert_matches_type(DatasetGetResponse, dataset, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_path_params_get(self, client: PremAI) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `dataset_id` but received ''"):
            client.datasets.with_raw_response.get(
                "",
            )


class TestAsyncDatasets:
    parametrize = pytest.mark.parametrize(
        "async_client", [False, True, {"http_client": "aiohttp"}], indirect=True, ids=["loose", "strict", "aiohttp"]
    )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_add_datapoint(self, async_client: AsyncPremAI) -> None:
        dataset = await async_client.datasets.add_datapoint(
            dataset_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            messages=[
                {
                    "content": "content",
                    "role": "user",
                }
            ],
        )
        assert_matches_type(DatasetAddDatapointResponse, dataset, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_add_datapoint_with_all_params(self, async_client: AsyncPremAI) -> None:
        dataset = await async_client.datasets.add_datapoint(
            dataset_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            messages=[
                {
                    "content": "content",
                    "role": "user",
                }
            ],
            bucket="uncategorized",
        )
        assert_matches_type(DatasetAddDatapointResponse, dataset, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_raw_response_add_datapoint(self, async_client: AsyncPremAI) -> None:
        response = await async_client.datasets.with_raw_response.add_datapoint(
            dataset_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            messages=[
                {
                    "content": "content",
                    "role": "user",
                }
            ],
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        dataset = await response.parse()
        assert_matches_type(DatasetAddDatapointResponse, dataset, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_streaming_response_add_datapoint(self, async_client: AsyncPremAI) -> None:
        async with async_client.datasets.with_streaming_response.add_datapoint(
            dataset_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            messages=[
                {
                    "content": "content",
                    "role": "user",
                }
            ],
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            dataset = await response.parse()
            assert_matches_type(DatasetAddDatapointResponse, dataset, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_path_params_add_datapoint(self, async_client: AsyncPremAI) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `dataset_id` but received ''"):
            await async_client.datasets.with_raw_response.add_datapoint(
                dataset_id="",
                messages=[
                    {
                        "content": "content",
                        "role": "user",
                    }
                ],
            )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_create_from_jsonl(self, async_client: AsyncPremAI) -> None:
        dataset = await async_client.datasets.create_from_jsonl(
            file=b"raw file contents",
            name="x",
            project_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        )
        assert_matches_type(DatasetCreateFromJSONLResponse, dataset, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_raw_response_create_from_jsonl(self, async_client: AsyncPremAI) -> None:
        response = await async_client.datasets.with_raw_response.create_from_jsonl(
            file=b"raw file contents",
            name="x",
            project_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        dataset = await response.parse()
        assert_matches_type(DatasetCreateFromJSONLResponse, dataset, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_streaming_response_create_from_jsonl(self, async_client: AsyncPremAI) -> None:
        async with async_client.datasets.with_streaming_response.create_from_jsonl(
            file=b"raw file contents",
            name="x",
            project_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            dataset = await response.parse()
            assert_matches_type(DatasetCreateFromJSONLResponse, dataset, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_create_synthetic(self, async_client: AsyncPremAI) -> None:
        dataset = await async_client.datasets.create_synthetic(
            name="x",
            pairs_to_generate=1,
            project_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        )
        assert_matches_type(DatasetCreateSyntheticResponse, dataset, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_create_synthetic_with_all_params(self, async_client: AsyncPremAI) -> None:
        dataset = await async_client.datasets.create_synthetic(
            name="x",
            pairs_to_generate=1,
            project_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            answer_format="answer_format",
            example_answers=["string"],
            example_questions=["string"],
            files=[b"raw file contents"],
            question_format="question_format",
            rules=["string"],
            temperature=0,
            website_urls=["https://example.com"],
            youtube_urls=["https://example.com"],
        )
        assert_matches_type(DatasetCreateSyntheticResponse, dataset, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_raw_response_create_synthetic(self, async_client: AsyncPremAI) -> None:
        response = await async_client.datasets.with_raw_response.create_synthetic(
            name="x",
            pairs_to_generate=1,
            project_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        dataset = await response.parse()
        assert_matches_type(DatasetCreateSyntheticResponse, dataset, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_streaming_response_create_synthetic(self, async_client: AsyncPremAI) -> None:
        async with async_client.datasets.with_streaming_response.create_synthetic(
            name="x",
            pairs_to_generate=1,
            project_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            dataset = await response.parse()
            assert_matches_type(DatasetCreateSyntheticResponse, dataset, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_get(self, async_client: AsyncPremAI) -> None:
        dataset = await async_client.datasets.get(
            "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        )
        assert_matches_type(DatasetGetResponse, dataset, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_raw_response_get(self, async_client: AsyncPremAI) -> None:
        response = await async_client.datasets.with_raw_response.get(
            "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        dataset = await response.parse()
        assert_matches_type(DatasetGetResponse, dataset, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_streaming_response_get(self, async_client: AsyncPremAI) -> None:
        async with async_client.datasets.with_streaming_response.get(
            "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            dataset = await response.parse()
            assert_matches_type(DatasetGetResponse, dataset, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_path_params_get(self, async_client: AsyncPremAI) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `dataset_id` but received ''"):
            await async_client.datasets.with_raw_response.get(
                "",
            )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from .model_load_params import ModelLoadParams as ModelLoadParams
from .model_list_response import ModelListResponse as ModelListResponse
from .model_load_response import ModelLoadResponse as ModelLoadResponse
from .model_unload_params import ModelUnloadParams as ModelUnloadParams
from .dataset_get_response import DatasetGetResponse as DatasetGetResponse
from .model_unload_response import ModelUnloadResponse as ModelUnloadResponse
from .project_create_params import ProjectCreateParams as ProjectCreateParams
from .project_list_response import ProjectListResponse as ProjectListResponse
from .snapshot_create_params import SnapshotCreateParams as SnapshotCreateParams
from .chat_completions_params import ChatCompletionsParams as ChatCompletionsParams
from .finetuning_get_response import FinetuningGetResponse as FinetuningGetResponse
from .project_create_response import ProjectCreateResponse as ProjectCreateResponse
from .finetuning_create_params import FinetuningCreateParams as FinetuningCreateParams
from .snapshot_create_response import SnapshotCreateResponse as SnapshotCreateResponse
from .chat_completions_response import ChatCompletionsResponse as ChatCompletionsResponse
from .model_check_status_params import ModelCheckStatusParams as ModelCheckStatusParams
from .project_get_tree_response import ProjectGetTreeResponse as ProjectGetTreeResponse
from .finetuning_create_response import FinetuningCreateResponse as FinetuningCreateResponse
from .model_check_status_response import ModelCheckStatusResponse as ModelCheckStatusResponse
from .recommendation_get_response import RecommendationGetResponse as RecommendationGetResponse
from .dataset_add_datapoint_params import DatasetAddDatapointParams as DatasetAddDatapointParams
from .dataset_add_datapoint_response import DatasetAddDatapointResponse as DatasetAddDatapointResponse
from .recommendation_generate_params import RecommendationGenerateParams as RecommendationGenerateParams
from .dataset_create_synthetic_params import DatasetCreateSyntheticParams as DatasetCreateSyntheticParams
from .dataset_create_from_jsonl_params import DatasetCreateFromJSONLParams as DatasetCreateFromJSONLParams
from .recommendation_generate_response import RecommendationGenerateResponse as RecommendationGenerateResponse
from .dataset_create_synthetic_response import DatasetCreateSyntheticResponse as DatasetCreateSyntheticResponse
from .dataset_create_from_jsonl_response import DatasetCreateFromJSONLResponse as DatasetCreateFromJSONLResponse



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import os
from typing import Any, cast

import pytest

from premai import PremAI, AsyncPremAI
from tests.utils import assert_matches_type
from premai.types import FinetuningGetResponse, FinetuningCreateResponse

base_url = os.environ.get("TEST_API_BASE_URL", "http://127.0.0.1:4010")


class TestFinetuning:
    parametrize = pytest.mark.parametrize("client", [False, True], indirect=True, ids=["loose", "strict"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_create(self, client: PremAI) -> None:
        finetuning = client.finetuning.create(
            experiments=[
                {
                    "base_model_id": "base_model_id",
                    "batch_size": 1,
                    "learning_rate_multiplier": 1,
                    "n_epochs": 1,
                }
            ],
            name="x",
            snapshot_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        )
        assert_matches_type(FinetuningCreateResponse, finetuning, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_raw_response_create(self, client: PremAI) -> None:
        response = client.finetuning.with_raw_response.create(
            experiments=[
                {
                    "base_model_id": "base_model_id",
                    "batch_size": 1,
                    "learning_rate_multiplier": 1,
                    "n_epochs": 1,
                }
            ],
            name="x",
            snapshot_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        finetuning = response.parse()
        assert_matches_type(FinetuningCreateResponse, finetuning, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_streaming_response_create(self, client: PremAI) -> None:
        with client.finetuning.with_streaming_response.create(
            experiments=[
                {
                    "base_model_id": "base_model_id",
                    "batch_size": 1,
                    "learning_rate_multiplier": 1,
                    "n_epochs": 1,
                }
            ],
            name="x",
            snapshot_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            finetuning = response.parse()
            assert_matches_type(FinetuningCreateResponse, finetuning, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_get(self, client: PremAI) -> None:
        finetuning = client.finetuning.get(
            "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        )
        assert_matches_type(FinetuningGetResponse, finetuning, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_raw_response_get(self, client: PremAI) -> None:
        response = client.finetuning.with_raw_response.get(
            "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        finetuning = response.parse()
        assert_matches_type(FinetuningGetResponse, finetuning, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_streaming_response_get(self, client: PremAI) -> None:
        with client.finetuning.with_streaming_response.get(
            "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            finetuning = response.parse()
            assert_matches_type(FinetuningGetResponse, finetuning, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_path_params_get(self, client: PremAI) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `job_id` but received ''"):
            client.finetuning.with_raw_response.get(
                "",
            )


class TestAsyncFinetuning:
    parametrize = pytest.mark.parametrize(
        "async_client", [False, True, {"http_client": "aiohttp"}], indirect=True, ids=["loose", "strict", "aiohttp"]
    )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_create(self, async_client: AsyncPremAI) -> None:
        finetuning = await async_client.finetuning.create(
            experiments=[
                {
                    "base_model_id": "base_model_id",
                    "batch_size": 1,
                    "learning_rate_multiplier": 1,
                    "n_epochs": 1,
                }
            ],
            name="x",
            snapshot_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        )
        assert_matches_type(FinetuningCreateResponse, finetuning, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_raw_response_create(self, async_client: AsyncPremAI) -> None:
        response = await async_client.finetuning.with_raw_response.create(
            experiments=[
                {
                    "base_model_id": "base_model_id",
                    "batch_size": 1,
                    "learning_rate_multiplier": 1,
                    "n_epochs": 1,
                }
            ],
            name="x",
            snapshot_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        finetuning = await response.parse()
        assert_matches_type(FinetuningCreateResponse, finetuning, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_streaming_response_create(self, async_client: AsyncPremAI) -> None:
        async with async_client.finetuning.with_streaming_response.create(
            experiments=[
                {
                    "base_model_id": "base_model_id",
                    "batch_size": 1,
                    "learning_rate_multiplier": 1,
                    "n_epochs": 1,
                }
            ],
            name="x",
            snapshot_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            finetuning = await response.parse()
            assert_matches_type(FinetuningCreateResponse, finetuning, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_get(self, async_client: AsyncPremAI) -> None:
        finetuning = await async_client.finetuning.get(
            "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        )
        assert_matches_type(FinetuningGetResponse, finetuning, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_raw_response_get(self, async_client: AsyncPremAI) -> None:
        response = await async_client.finetuning.with_raw_response.get(
            "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        finetuning = await response.parse()
        assert_matches_type(FinetuningGetResponse, finetuning, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_streaming_response_get(self, async_client: AsyncPremAI) -> None:
        async with async_client.finetuning.with_streaming_response.get(
            "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            finetuning = await response.parse()
            assert_matches_type(FinetuningGetResponse, finetuning, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_path_params_get(self, async_client: AsyncPremAI) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `job_id` but received ''"):
            await async_client.finetuning.with_raw_response.get(
                "",
            )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from .._models import BaseModel

__all__ = ["DatasetCreateFromJSONLResponse"]


class DatasetCreateFromJSONLResponse(BaseModel):
    dataset_id: str



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import httpx

from ..types import snapshot_create_params
from .._types import Body, Omit, Query, Headers, NotGiven, omit, not_given
from .._utils import maybe_transform, async_maybe_transform
from .._compat import cached_property
from .._resource import SyncAPIResource, AsyncAPIResource
from .._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from .._base_client import make_request_options
from ..types.snapshot_create_response import SnapshotCreateResponse

__all__ = ["SnapshotsResource", "AsyncSnapshotsResource"]


class SnapshotsResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> SnapshotsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/premAI-io/prem-py-sdk#accessing-raw-response-data-eg-headers
        """
        return SnapshotsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> SnapshotsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/premAI-io/prem-py-sdk#with_streaming_response
        """
        return SnapshotsResourceWithStreamingResponse(self)

    def create(
        self,
        *,
        dataset_id: str,
        split_percentage: int | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> SnapshotCreateResponse:
        """
        Create snapshot from dataset with train/validation split

        Args:
          dataset_id: Dataset ID to snapshot. The dataset must belong to the authenticated workspace.

          split_percentage: Percentage of datapoints to assign to training. Remaining datapoints go to
              validation.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._post(
            "/api/v1/public/snapshots/create",
            body=maybe_transform(
                {
                    "dataset_id": dataset_id,
                    "split_percentage": split_percentage,
                },
                snapshot_create_params.SnapshotCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=SnapshotCreateResponse,
        )


class AsyncSnapshotsResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncSnapshotsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/premAI-io/prem-py-sdk#accessing-raw-response-data-eg-headers
        """
        return AsyncSnapshotsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncSnapshotsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/premAI-io/prem-py-sdk#with_streaming_response
        """
        return AsyncSnapshotsResourceWithStreamingResponse(self)

    async def create(
        self,
        *,
        dataset_id: str,
        split_percentage: int | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> SnapshotCreateResponse:
        """
        Create snapshot from dataset with train/validation split

        Args:
          dataset_id: Dataset ID to snapshot. The dataset must belong to the authenticated workspace.

          split_percentage: Percentage of datapoints to assign to training. Remaining datapoints go to
              validation.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._post(
            "/api/v1/public/snapshots/create",
            body=await async_maybe_transform(
                {
                    "dataset_id": dataset_id,
                    "split_percentage": split_percentage,
                },
                snapshot_create_params.SnapshotCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=SnapshotCreateResponse,
        )


class SnapshotsResourceWithRawResponse:
    def __init__(self, snapshots: SnapshotsResource) -> None:
        self._snapshots = snapshots

        self.create = to_raw_response_wrapper(
            snapshots.create,
        )


class AsyncSnapshotsResourceWithRawResponse:
    def __init__(self, snapshots: AsyncSnapshotsResource) -> None:
        self._snapshots = snapshots

        self.create = async_to_raw_response_wrapper(
            snapshots.create,
        )


class SnapshotsResourceWithStreamingResponse:
    def __init__(self, snapshots: SnapshotsResource) -> None:
        self._snapshots = snapshots

        self.create = to_streamed_response_wrapper(
            snapshots.create,
        )


class AsyncSnapshotsResourceWithStreamingResponse:
    def __init__(self, snapshots: AsyncSnapshotsResource) -> None:
        self._snapshots = snapshots

        self.create = async_to_streamed_response_wrapper(
            snapshots.create,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List

from .._models import BaseModel

__all__ = ["ProjectListResponse", "Project"]


class Project(BaseModel):
    id: str

    created_at: str

    name: str


class ProjectListResponse(BaseModel):
    projects: List[Project]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import os
from typing import Any, Mapping
from typing_extensions import Self, override

import httpx

from . import _exceptions
from ._qs import Querystring
from ._types import (
    Omit,
    Timeout,
    NotGiven,
    Transport,
    ProxiesTypes,
    RequestOptions,
    not_given,
)
from ._utils import is_given, get_async_library
from ._version import __version__
from .resources import chat, models, datasets, projects, snapshots, finetuning, recommendations
from ._streaming import Stream as Stream, AsyncStream as AsyncStream
from ._exceptions import PremAIError, APIStatusError
from ._base_client import (
    DEFAULT_MAX_RETRIES,
    SyncAPIClient,
    AsyncAPIClient,
)

__all__ = ["Timeout", "Transport", "ProxiesTypes", "RequestOptions", "PremAI", "AsyncPremAI", "Client", "AsyncClient"]


class PremAI(SyncAPIClient):
    chat: chat.ChatResource
    models: models.ModelsResource
    projects: projects.ProjectsResource
    datasets: datasets.DatasetsResource
    snapshots: snapshots.SnapshotsResource
    recommendations: recommendations.RecommendationsResource
    finetuning: finetuning.FinetuningResource
    with_raw_response: PremAIWithRawResponse
    with_streaming_response: PremAIWithStreamedResponse

    # client options
    api_key: str

    def __init__(
        self,
        *,
        api_key: str | None = None,
        base_url: str | httpx.URL | None = None,
        timeout: float | Timeout | None | NotGiven = not_given,
        max_retries: int = DEFAULT_MAX_RETRIES,
        default_headers: Mapping[str, str] | None = None,
        default_query: Mapping[str, object] | None = None,
        # Configure a custom httpx client.
        # We provide a `DefaultHttpxClient` class that you can pass to retain the default values we use for `limits`, `timeout` & `follow_redirects`.
        # See the [httpx documentation](https://www.python-httpx.org/api/#client) for more details.
        http_client: httpx.Client | None = None,
        # Enable or disable schema validation for data returned by the API.
        # When enabled an error APIResponseValidationError is raised
        # if the API responds with invalid data for the expected schema.
        #
        # This parameter may be removed or changed in the future.
        # If you rely on this feature, please open a GitHub issue
        # outlining your use-case to help us decide if it should be
        # part of our public interface in the future.
        _strict_response_validation: bool = False,
    ) -> None:
        """Construct a new synchronous PremAI client instance.

        This automatically infers the `api_key` argument from the `PREMAI_API_KEY` environment variable if it is not provided.
        """
        if api_key is None:
            api_key = os.environ.get("PREMAI_API_KEY")
        if api_key is None:
            raise PremAIError(
                "The api_key client option must be set either by passing api_key to the client or by setting the PREMAI_API_KEY environment variable"
            )
        self.api_key = api_key

        if base_url is None:
            base_url = os.environ.get("PREM_AI_BASE_URL")
        if base_url is None:
            base_url = f"http://localhost:3000"

        super().__init__(
            version=__version__,
            base_url=base_url,
            max_retries=max_retries,
            timeout=timeout,
            http_client=http_client,
            custom_headers=default_headers,
            custom_query=default_query,
            _strict_response_validation=_strict_response_validation,
        )

        self.chat = chat.ChatResource(self)
        self.models = models.ModelsResource(self)
        self.projects = projects.ProjectsResource(self)
        self.datasets = datasets.DatasetsResource(self)
        self.snapshots = snapshots.SnapshotsResource(self)
        self.recommendations = recommendations.RecommendationsResource(self)
        self.finetuning = finetuning.FinetuningResource(self)
        self.with_raw_response = PremAIWithRawResponse(self)
        self.with_streaming_response = PremAIWithStreamedResponse(self)

    @property
    @override
    def qs(self) -> Querystring:
        return Querystring(array_format="comma")

    @property
    @override
    def auth_headers(self) -> dict[str, str]:
        api_key = self.api_key
        return {"Authorization": f"Bearer {api_key}"}

    @property
    @override
    def default_headers(self) -> dict[str, str | Omit]:
        return {
            **super().default_headers,
            "X-Stainless-Async": "false",
            **self._custom_headers,
        }

    def copy(
        self,
        *,
        api_key: str | None = None,
        base_url: str | httpx.URL | None = None,
        timeout: float | Timeout | None | NotGiven = not_given,
        http_client: httpx.Client | None = None,
        max_retries: int | NotGiven = not_given,
        default_headers: Mapping[str, str] | None = None,
        set_default_headers: Mapping[str, str] | None = None,
        default_query: Mapping[str, object] | None = None,
        set_default_query: Mapping[str, object] | None = None,
        _extra_kwargs: Mapping[str, Any] = {},
    ) -> Self:
        """
        Create a new client instance re-using the same options given to the current client with optional overriding.
        """
        if default_headers is not None and set_default_headers is not None:
            raise ValueError("The `default_headers` and `set_default_headers` arguments are mutually exclusive")

        if default_query is not None and set_default_query is not None:
            raise ValueError("The `default_query` and `set_default_query` arguments are mutually exclusive")

        headers = self._custom_headers
        if default_headers is not None:
            headers = {**headers, **default_headers}
        elif set_default_headers is not None:
            headers = set_default_headers

        params = self._custom_query
        if default_query is not None:
            params = {**params, **default_query}
        elif set_default_query is not None:
            params = set_default_query

        http_client = http_client or self._client
        return self.__class__(
            api_key=api_key or self.api_key,
            base_url=base_url or self.base_url,
            timeout=self.timeout if isinstance(timeout, NotGiven) else timeout,
            http_client=http_client,
            max_retries=max_retries if is_given(max_retries) else self.max_retries,
            default_headers=headers,
            default_query=params,
            **_extra_kwargs,
        )

    # Alias for `copy` for nicer inline usage, e.g.
    # client.with_options(timeout=10).foo.create(...)
    with_options = copy

    @override
    def _make_status_error(
        self,
        err_msg: str,
        *,
        body: object,
        response: httpx.Response,
    ) -> APIStatusError:
        if response.status_code == 400:
            return _exceptions.BadRequestError(err_msg, response=response, body=body)

        if response.status_code == 401:
            return _exceptions.AuthenticationError(err_msg, response=response, body=body)

        if response.status_code == 403:
            return _exceptions.PermissionDeniedError(err_msg, response=response, body=body)

        if response.status_code == 404:
            return _exceptions.NotFoundError(err_msg, response=response, body=body)

        if response.status_code == 409:
            return _exceptions.ConflictError(err_msg, response=response, body=body)

        if response.status_code == 422:
            return _exceptions.UnprocessableEntityError(err_msg, response=response, body=body)

        if response.status_code == 429:
            return _exceptions.RateLimitError(err_msg, response=response, body=body)

        if response.status_code >= 500:
            return _exceptions.InternalServerError(err_msg, response=response, body=body)
        return APIStatusError(err_msg, response=response, body=body)


class AsyncPremAI(AsyncAPIClient):
    chat: chat.AsyncChatResource
    models: models.AsyncModelsResource
    projects: projects.AsyncProjectsResource
    datasets: datasets.AsyncDatasetsResource
    snapshots: snapshots.AsyncSnapshotsResource
    recommendations: recommendations.AsyncRecommendationsResource
    finetuning: finetuning.AsyncFinetuningResource
    with_raw_response: AsyncPremAIWithRawResponse
    with_streaming_response: AsyncPremAIWithStreamedResponse

    # client options
    api_key: str

    def __init__(
        self,
        *,
        api_key: str | None = None,
        base_url: str | httpx.URL | None = None,
        timeout: float | Timeout | None | NotGiven = not_given,
        max_retries: int = DEFAULT_MAX_RETRIES,
        default_headers: Mapping[str, str] | None = None,
        default_query: Mapping[str, object] | None = None,
        # Configure a custom httpx client.
        # We provide a `DefaultAsyncHttpxClient` class that you can pass to retain the default values we use for `limits`, `timeout` & `follow_redirects`.
        # See the [httpx documentation](https://www.python-httpx.org/api/#asyncclient) for more details.
        http_client: httpx.AsyncClient | None = None,
        # Enable or disable schema validation for data returned by the API.
        # When enabled an error APIResponseValidationError is raised
        # if the API responds with invalid data for the expected schema.
        #
        # This parameter may be removed or changed in the future.
        # If you rely on this feature, please open a GitHub issue
        # outlining your use-case to help us decide if it should be
        # part of our public interface in the future.
        _strict_response_validation: bool = False,
    ) -> None:
        """Construct a new async AsyncPremAI client instance.

        This automatically infers the `api_key` argument from the `PREMAI_API_KEY` environment variable if it is not provided.
        """
        if api_key is None:
            api_key = os.environ.get("PREMAI_API_KEY")
        if api_key is None:
            raise PremAIError(
                "The api_key client option must be set either by passing api_key to the client or by setting the PREMAI_API_KEY environment variable"
            )
        self.api_key = api_key

        if base_url is None:
            base_url = os.environ.get("PREM_AI_BASE_URL")
        if base_url is None:
            base_url = f"http://localhost:3000"

        super().__init__(
            version=__version__,
            base_url=base_url,
            max_retries=max_retries,
            timeout=timeout,
            http_client=http_client,
            custom_headers=default_headers,
            custom_query=default_query,
            _strict_response_validation=_strict_response_validation,
        )

        self.chat = chat.AsyncChatResource(self)
        self.models = models.AsyncModelsResource(self)
        self.projects = projects.AsyncProjectsResource(self)
        self.datasets = datasets.AsyncDatasetsResource(self)
        self.snapshots = snapshots.AsyncSnapshotsResource(self)
        self.recommendations = recommendations.AsyncRecommendationsResource(self)
        self.finetuning = finetuning.AsyncFinetuningResource(self)
        self.with_raw_response = AsyncPremAIWithRawResponse(self)
        self.with_streaming_response = AsyncPremAIWithStreamedResponse(self)

    @property
    @override
    def qs(self) -> Querystring:
        return Querystring(array_format="comma")

    @property
    @override
    def auth_headers(self) -> dict[str, str]:
        api_key = self.api_key
        return {"Authorization": f"Bearer {api_key}"}

    @property
    @override
    def default_headers(self) -> dict[str, str | Omit]:
        return {
            **super().default_headers,
            "X-Stainless-Async": f"async:{get_async_library()}",
            **self._custom_headers,
        }

    def copy(
        self,
        *,
        api_key: str | None = None,
        base_url: str | httpx.URL | None = None,
        timeout: float | Timeout | None | NotGiven = not_given,
        http_client: httpx.AsyncClient | None = None,
        max_retries: int | NotGiven = not_given,
        default_headers: Mapping[str, str] | None = None,
        set_default_headers: Mapping[str, str] | None = None,
        default_query: Mapping[str, object] | None = None,
        set_default_query: Mapping[str, object] | None = None,
        _extra_kwargs: Mapping[str, Any] = {},
    ) -> Self:
        """
        Create a new client instance re-using the same options given to the current client with optional overriding.
        """
        if default_headers is not None and set_default_headers is not None:
            raise ValueError("The `default_headers` and `set_default_headers` arguments are mutually exclusive")

        if default_query is not None and set_default_query is not None:
            raise ValueError("The `default_query` and `set_default_query` arguments are mutually exclusive")

        headers = self._custom_headers
        if default_headers is not None:
            headers = {**headers, **default_headers}
        elif set_default_headers is not None:
            headers = set_default_headers

        params = self._custom_query
        if default_query is not None:
            params = {**params, **default_query}
        elif set_default_query is not None:
            params = set_default_query

        http_client = http_client or self._client
        return self.__class__(
            api_key=api_key or self.api_key,
            base_url=base_url or self.base_url,
            timeout=self.timeout if isinstance(timeout, NotGiven) else timeout,
            http_client=http_client,
            max_retries=max_retries if is_given(max_retries) else self.max_retries,
            default_headers=headers,
            default_query=params,
            **_extra_kwargs,
        )

    # Alias for `copy` for nicer inline usage, e.g.
    # client.with_options(timeout=10).foo.create(...)
    with_options = copy

    @override
    def _make_status_error(
        self,
        err_msg: str,
        *,
        body: object,
        response: httpx.Response,
    ) -> APIStatusError:
        if response.status_code == 400:
            return _exceptions.BadRequestError(err_msg, response=response, body=body)

        if response.status_code == 401:
            return _exceptions.AuthenticationError(err_msg, response=response, body=body)

        if response.status_code == 403:
            return _exceptions.PermissionDeniedError(err_msg, response=response, body=body)

        if response.status_code == 404:
            return _exceptions.NotFoundError(err_msg, response=response, body=body)

        if response.status_code == 409:
            return _exceptions.ConflictError(err_msg, response=response, body=body)

        if response.status_code == 422:
            return _exceptions.UnprocessableEntityError(err_msg, response=response, body=body)

        if response.status_code == 429:
            return _exceptions.RateLimitError(err_msg, response=response, body=body)

        if response.status_code >= 500:
            return _exceptions.InternalServerError(err_msg, response=response, body=body)
        return APIStatusError(err_msg, response=response, body=body)


class PremAIWithRawResponse:
    def __init__(self, client: PremAI) -> None:
        self.chat = chat.ChatResourceWithRawResponse(client.chat)
        self.models = models.ModelsResourceWithRawResponse(client.models)
        self.projects = projects.ProjectsResourceWithRawResponse(client.projects)
        self.datasets = datasets.DatasetsResourceWithRawResponse(client.datasets)
        self.snapshots = snapshots.SnapshotsResourceWithRawResponse(client.snapshots)
        self.recommendations = recommendations.RecommendationsResourceWithRawResponse(client.recommendations)
        self.finetuning = finetuning.FinetuningResourceWithRawResponse(client.finetuning)


class AsyncPremAIWithRawResponse:
    def __init__(self, client: AsyncPremAI) -> None:
        self.chat = chat.AsyncChatResourceWithRawResponse(client.chat)
        self.models = models.AsyncModelsResourceWithRawResponse(client.models)
        self.projects = projects.AsyncProjectsResourceWithRawResponse(client.projects)
        self.datasets = datasets.AsyncDatasetsResourceWithRawResponse(client.datasets)
        self.snapshots = snapshots.AsyncSnapshotsResourceWithRawResponse(client.snapshots)
        self.recommendations = recommendations.AsyncRecommendationsResourceWithRawResponse(client.recommendations)
        self.finetuning = finetuning.AsyncFinetuningResourceWithRawResponse(client.finetuning)


class PremAIWithStreamedResponse:
    def __init__(self, client: PremAI) -> None:
        self.chat = chat.ChatResourceWithStreamingResponse(client.chat)
        self.models = models.ModelsResourceWithStreamingResponse(client.models)
        self.projects = projects.ProjectsResourceWithStreamingResponse(client.projects)
        self.datasets = datasets.DatasetsResourceWithStreamingResponse(client.datasets)
        self.snapshots = snapshots.SnapshotsResourceWithStreamingResponse(client.snapshots)
        self.recommendations = recommendations.RecommendationsResourceWithStreamingResponse(client.recommendations)
        self.finetuning = finetuning.FinetuningResourceWithStreamingResponse(client.finetuning)


class AsyncPremAIWithStreamedResponse:
    def __init__(self, client: AsyncPremAI) -> None:
        self.chat = chat.AsyncChatResourceWithStreamingResponse(client.chat)
        self.models = models.AsyncModelsResourceWithStreamingResponse(client.models)
        self.projects = projects.AsyncProjectsResourceWithStreamingResponse(client.projects)
        self.datasets = datasets.AsyncDatasetsResourceWithStreamingResponse(client.datasets)
        self.snapshots = snapshots.AsyncSnapshotsResourceWithStreamingResponse(client.snapshots)
        self.recommendations = recommendations.AsyncRecommendationsResourceWithStreamingResponse(client.recommendations)
        self.finetuning = finetuning.AsyncFinetuningResourceWithStreamingResponse(client.finetuning)


Client = PremAI

AsyncClient = AsyncPremAI



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Iterable, Optional
from typing_extensions import Required, TypedDict

__all__ = ["FinetuningCreateParams", "Experiment"]


class FinetuningCreateParams(TypedDict, total=False):
    experiments: Required[Iterable[Experiment]]

    name: Required[str]

    snapshot_id: Required[str]


class Experiment(TypedDict, total=False):
    base_model_id: Required[str]

    batch_size: Required[int]

    learning_rate_multiplier: Required[float]

    n_epochs: Required[int]

    lora: Optional[bool]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from .._models import BaseModel

__all__ = ["DatasetAddDatapointResponse"]


class DatasetAddDatapointResponse(BaseModel):
    id: str

    message: str



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Optional
from typing_extensions import Required, TypedDict

from .._types import FileTypes, SequenceNotStr

__all__ = ["DatasetCreateSyntheticParams"]


class DatasetCreateSyntheticParams(TypedDict, total=False):
    name: Required[str]

    pairs_to_generate: Required[int]

    project_id: Required[str]

    answer_format: str
    """Answer format template"""

    example_answers: SequenceNotStr[str]
    """Example answers"""

    example_questions: SequenceNotStr[str]
    """Example questions"""

    files: SequenceNotStr[FileTypes]
    """Optional: PDF, DOCX, etc."""

    question_format: str
    """Question format template"""

    rules: SequenceNotStr[str]
    """Array of rules and constraints"""

    temperature: Optional[float]
    """0.0-1.0, controls randomness"""

    website_urls: SequenceNotStr[str]
    """Array of website URLs"""

    youtube_urls: SequenceNotStr[str]
    """Array of YouTube URLs"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional
from typing_extensions import Literal

from .._models import BaseModel

__all__ = ["DatasetGetResponse"]


class DatasetGetResponse(BaseModel):
    id: str

    created_at: str

    datapoints_count: int

    name: str

    project_id: Optional[str] = None

    status: Literal["processing", "completed", "failed"]

    updated_at: str



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import httpx

from ..types import recommendation_generate_params
from .._types import Body, Query, Headers, NotGiven, not_given
from .._utils import maybe_transform, async_maybe_transform
from .._compat import cached_property
from .._resource import SyncAPIResource, AsyncAPIResource
from .._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from .._base_client import make_request_options
from ..types.recommendation_get_response import RecommendationGetResponse
from ..types.recommendation_generate_response import RecommendationGenerateResponse

__all__ = ["RecommendationsResource", "AsyncRecommendationsResource"]


class RecommendationsResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> RecommendationsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/premAI-io/prem-py-sdk#accessing-raw-response-data-eg-headers
        """
        return RecommendationsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> RecommendationsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/premAI-io/prem-py-sdk#with_streaming_response
        """
        return RecommendationsResourceWithStreamingResponse(self)

    def generate(
        self,
        *,
        snapshot_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> RecommendationGenerateResponse:
        """
        Generate model recommendations for a snapshot

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._post(
            "/api/v1/public/recommendations/generate",
            body=maybe_transform(
                {"snapshot_id": snapshot_id}, recommendation_generate_params.RecommendationGenerateParams
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=RecommendationGenerateResponse,
        )

    def get(
        self,
        snapshot_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> RecommendationGetResponse:
        """
        Get recommendations status for a snapshot

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not snapshot_id:
            raise ValueError(f"Expected a non-empty value for `snapshot_id` but received {snapshot_id!r}")
        return self._get(
            f"/api/v1/public/recommendations/{snapshot_id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=RecommendationGetResponse,
        )


class AsyncRecommendationsResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncRecommendationsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/premAI-io/prem-py-sdk#accessing-raw-response-data-eg-headers
        """
        return AsyncRecommendationsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncRecommendationsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/premAI-io/prem-py-sdk#with_streaming_response
        """
        return AsyncRecommendationsResourceWithStreamingResponse(self)

    async def generate(
        self,
        *,
        snapshot_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> RecommendationGenerateResponse:
        """
        Generate model recommendations for a snapshot

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._post(
            "/api/v1/public/recommendations/generate",
            body=await async_maybe_transform(
                {"snapshot_id": snapshot_id}, recommendation_generate_params.RecommendationGenerateParams
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=RecommendationGenerateResponse,
        )

    async def get(
        self,
        snapshot_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> RecommendationGetResponse:
        """
        Get recommendations status for a snapshot

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not snapshot_id:
            raise ValueError(f"Expected a non-empty value for `snapshot_id` but received {snapshot_id!r}")
        return await self._get(
            f"/api/v1/public/recommendations/{snapshot_id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=RecommendationGetResponse,
        )


class RecommendationsResourceWithRawResponse:
    def __init__(self, recommendations: RecommendationsResource) -> None:
        self._recommendations = recommendations

        self.generate = to_raw_response_wrapper(
            recommendations.generate,
        )
        self.get = to_raw_response_wrapper(
            recommendations.get,
        )


class AsyncRecommendationsResourceWithRawResponse:
    def __init__(self, recommendations: AsyncRecommendationsResource) -> None:
        self._recommendations = recommendations

        self.generate = async_to_raw_response_wrapper(
            recommendations.generate,
        )
        self.get = async_to_raw_response_wrapper(
            recommendations.get,
        )


class RecommendationsResourceWithStreamingResponse:
    def __init__(self, recommendations: RecommendationsResource) -> None:
        self._recommendations = recommendations

        self.generate = to_streamed_response_wrapper(
            recommendations.generate,
        )
        self.get = to_streamed_response_wrapper(
            recommendations.get,
        )


class AsyncRecommendationsResourceWithStreamingResponse:
    def __init__(self, recommendations: AsyncRecommendationsResource) -> None:
        self._recommendations = recommendations

        self.generate = async_to_streamed_response_wrapper(
            recommendations.generate,
        )
        self.get = async_to_streamed_response_wrapper(
            recommendations.get,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Required, TypedDict

__all__ = ["SnapshotCreateParams"]


class SnapshotCreateParams(TypedDict, total=False):
    dataset_id: Required[str]
    """Dataset ID to snapshot. The dataset must belong to the authenticated workspace."""

    split_percentage: int
    """Percentage of datapoints to assign to training.

    Remaining datapoints go to validation.
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Iterable

import httpx

from ..types import finetuning_create_params
from .._types import Body, Query, Headers, NotGiven, not_given
from .._utils import maybe_transform, async_maybe_transform
from .._compat import cached_property
from .._resource import SyncAPIResource, AsyncAPIResource
from .._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from .._base_client import make_request_options
from ..types.finetuning_get_response import FinetuningGetResponse
from ..types.finetuning_create_response import FinetuningCreateResponse

__all__ = ["FinetuningResource", "AsyncFinetuningResource"]


class FinetuningResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> FinetuningResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/premAI-io/prem-py-sdk#accessing-raw-response-data-eg-headers
        """
        return FinetuningResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> FinetuningResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/premAI-io/prem-py-sdk#with_streaming_response
        """
        return FinetuningResourceWithStreamingResponse(self)

    def create(
        self,
        *,
        experiments: Iterable[finetuning_create_params.Experiment],
        name: str,
        snapshot_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> FinetuningCreateResponse:
        """
        Create and start fine-tuning job

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._post(
            "/api/v1/public/finetuning/create",
            body=maybe_transform(
                {
                    "experiments": experiments,
                    "name": name,
                    "snapshot_id": snapshot_id,
                },
                finetuning_create_params.FinetuningCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=FinetuningCreateResponse,
        )

    def get(
        self,
        job_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> FinetuningGetResponse:
        """
        Get fine-tuning job status

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not job_id:
            raise ValueError(f"Expected a non-empty value for `job_id` but received {job_id!r}")
        return self._get(
            f"/api/v1/public/finetuning/{job_id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=FinetuningGetResponse,
        )


class AsyncFinetuningResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncFinetuningResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/premAI-io/prem-py-sdk#accessing-raw-response-data-eg-headers
        """
        return AsyncFinetuningResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncFinetuningResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/premAI-io/prem-py-sdk#with_streaming_response
        """
        return AsyncFinetuningResourceWithStreamingResponse(self)

    async def create(
        self,
        *,
        experiments: Iterable[finetuning_create_params.Experiment],
        name: str,
        snapshot_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> FinetuningCreateResponse:
        """
        Create and start fine-tuning job

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._post(
            "/api/v1/public/finetuning/create",
            body=await async_maybe_transform(
                {
                    "experiments": experiments,
                    "name": name,
                    "snapshot_id": snapshot_id,
                },
                finetuning_create_params.FinetuningCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=FinetuningCreateResponse,
        )

    async def get(
        self,
        job_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> FinetuningGetResponse:
        """
        Get fine-tuning job status

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not job_id:
            raise ValueError(f"Expected a non-empty value for `job_id` but received {job_id!r}")
        return await self._get(
            f"/api/v1/public/finetuning/{job_id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=FinetuningGetResponse,
        )


class FinetuningResourceWithRawResponse:
    def __init__(self, finetuning: FinetuningResource) -> None:
        self._finetuning = finetuning

        self.create = to_raw_response_wrapper(
            finetuning.create,
        )
        self.get = to_raw_response_wrapper(
            finetuning.get,
        )


class AsyncFinetuningResourceWithRawResponse:
    def __init__(self, finetuning: AsyncFinetuningResource) -> None:
        self._finetuning = finetuning

        self.create = async_to_raw_response_wrapper(
            finetuning.create,
        )
        self.get = async_to_raw_response_wrapper(
            finetuning.get,
        )


class FinetuningResourceWithStreamingResponse:
    def __init__(self, finetuning: FinetuningResource) -> None:
        self._finetuning = finetuning

        self.create = to_streamed_response_wrapper(
            finetuning.create,
        )
        self.get = to_streamed_response_wrapper(
            finetuning.get,
        )


class AsyncFinetuningResourceWithStreamingResponse:
    def __init__(self, finetuning: AsyncFinetuningResource) -> None:
        self._finetuning = finetuning

        self.create = async_to_streamed_response_wrapper(
            finetuning.create,
        )
        self.get = async_to_streamed_response_wrapper(
            finetuning.get,
        )
